package net.javajigi.advice;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.aspectj.lang.Signature;

public aspect LoggingAspect {
	protected final Log logger = LogFactory.getLog(getClass());

	protected pointcut loggedOperations()
        : (execution(public * net.javajigi.user.dao..*.*(..)) 
                || execution(public * net.javajigi.user.service.UserServiceImpl*.*(..)) );

	before() : loggedOperations(){
		Signature sig = thisJoinPointStaticPart.getSignature();

		if (logger.isDebugEnabled()) {
			logger.debug("" + sig.getDeclaringTypeName() + "." + sig.getName()
					+ "()" + " ");

			Object[] arguments = thisJoinPoint.getArgs();
			if (logger.isDebugEnabled()) {
				for (int length = arguments.length, i = 0; i < length; ++i) {
					logger.debug("Argument[" + i + "] : " + arguments[i]);
				}
			}
		}
	}

	after() :  loggedOperations (){
		Signature sig = thisJoinPointStaticPart.getSignature();

		if (logger.isDebugEnabled()) {
			logger.debug("" + sig.getDeclaringTypeName() + "." + sig.getName()
					+ " ");
		}
	}
}
