/**
 *  ҽ Spring ӿũ ũϿ   ҽԴϴ. 
 *  ҽ  ڵ Ӱ     ֽϴ. 
 * ,  ҽ  ο ø̼   ó  ֽø ˴ϴ. 
 */
package net.javajigi.user.service;

import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.BeansException;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;

import net.javajigi.user.ExistedUserException;
import net.javajigi.user.PasswordMismatchException;
import net.javajigi.user.UserNotFoundException;
import net.javajigi.user.dao.UserDAO;
import net.javajigi.user.model.User;

/**
 * ּ ־ ּ. 谡 Ŀ.
 * 
 * @author 缺(ڹ, javajigi@gmail.com)
 */
public class UserServiceImpl implements UserService, InitializingBean,
		ApplicationContextAware {
	protected final Log logger = LogFactory.getLog(getClass());

	private UserDAO userDAO;

	private ApplicationContext context;

	public void afterPropertiesSet() throws Exception {
		if (context == null) {
			throw new Exception("ApplicationContext ʱȭ ʾҽϴ.");
		}

		if (userDAO == null) {
			throw new Exception(context.getMessage("instance.not.init",
					new Object[] { "UserDAO" }, null));
		}
	}

	public void setUserDAO(UserDAO newUserDAO) {
		this.userDAO = newUserDAO;
	}

	public void setApplicationContext(ApplicationContext context)
			throws BeansException {
		this.context = context;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.javajigi.user.spring.UserService#create(net.javajigi.user.model.User)
	 */
	public int addUser(User user) throws ExistedUserException {
		if (userDAO.existedUser(user.getUserId())) {
			throw new ExistedUserException(context.getMessage(
					"user.existed.exception",
					new Object[] { user.getUserId() }, null));
		}

		int result = userDAO.insert(user);

		if (logger.isDebugEnabled()) {
			logger.debug("Added " + user.getUserId() + " in User System!!");
		}

		return result;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.javajigi.user.spring.UserService#update(net.javajigi.user.model.User)
	 */
	public int updateUser(User user) {
		int result = userDAO.update(user);

		if (logger.isDebugEnabled()) {
			logger.debug("Updated " + user.getUserId() + " in User System!!");
		}

		return result;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.javajigi.user.spring.UserService#remove(java.lang.String)
	 */
	public int removeUser(String userId) {
		int result = userDAO.delete(userId);

		if (logger.isDebugEnabled()) {
			logger.debug("Removed " + userId + " in User System!!");
		}

		return result;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.javajigi.user.spring.UserService#findUser(java.lang.String)
	 */
	public User findUser(String userId) throws UserNotFoundException {
		User user = userDAO.findUser(userId);

		if (user == null) {
			throw new UserNotFoundException(context.getMessage(
					"user.notfound.exception", new Object[] { userId }, null));
		}

		if (logger.isDebugEnabled()) {
			logger.debug(userId + "   : " + user);
		}
		return user;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.javajigi.user.spring.UserService#findUserList()
	 */
	public List findUserList() {
		List userList = userDAO.findUserList();

		if (logger.isDebugEnabled()) {
			if (userList != null) {
				logger.debug("User System   : " + userList.size());
			} else {
				logger.debug("User System   : 0");
			}
		}

		return userList;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.javajigi.user.spring.UserService#login(java.lang.String,
	 *      java.lang.String)
	 */
	public boolean login(String userId, String password)
			throws PasswordMismatchException, UserNotFoundException {
		User user = findUser(userId);

		if (!user.isMatchPassword(password)) {
			throw new PasswordMismatchException(context.getMessage(
					"password.mismatch.exception", new Object[] { userId },
					null));
		}

		return true;
	}
}
